% This is the code used to generate results for
% "Causal Learning Without DAGs" by 
% David Duvenaud, Daniel Eaton, Mark Schmidt and Kevin murphy.
% 
% It includes versions of the following software packages:
% BDAGL, ( Bayesian DAG Learner )
% http://www.cs.ubc.ca/~murphyk/Software/BDAGL/
% 
% PMTK ( Probabilistic Modeling Toolkit )
% http://www.cs.ubc.ca/~murphyk/pmtk/
% 
% Minfunc
% http://www.cs.ubc.ca/~schmidtm/Software/minFunc.html
% 
% 
% To reproduce the figures generated by the paper, simply run
% produceFigures.m.
%
% If you have questions about this code, contact
% David Duvenaud ( duvenaud@cs.ubc.ca )
%
% call this file from the directory that produce_all_figures.m is in.
addpath(genpath(pwd));  % for PMTK and BDAGL

% indices of different models ( see main.m )
all_models = 4:12;
ignore = [ 4 7 10 ];
cond = [ 6 9 12 ];
perfect = [ 11 ];
ind = [ 5 8 ];
UGM = [ 7 8 9 ];
DAG = [ 10 11 12 ];

% indices of different datasets ( see main.m )
sachs = 2;
SEM_visible = 4;


% Produce Figure 2
% ==================================================

Nfolds = 10;
checkforold = 1;   % check for existing results files
models = all_models;
dataset = sachs;

% run all the experiments needed
main( models, dataset, Nfolds, checkforold );

% aggregate the results and display them
show_figure_2;



% Produce Figure 3
% ===================================================

Nfolds = 2;
checkforold = 1;   % check for existing results files
models = [ UGM DAG ];
dataset = sachs;
samples_unseen_action_list = [ 10 20 35 50 100 200 ];
action = 7;

% run all the experiments needed
for s = 1:length( samples_unseen_action_list )
    main( models, dataset, Nfolds, checkforold, samples_unseen_action_list( s ), action );
end

% aggregate the results and display them
show_figure_3;



% Produce Figure 4
% ===================================================

Nfolds = 2;
checkforold = 1;   % check for existing results files
models = [ ignore cond perfect ];
dataset = sachs;
samples_unseen_action = 0;    % don't show any training examples of the unseen action

% run all the experiments needed
for action = 1:8
    main( models, dataset, Nfolds, checkforold, samples_unseen_action, action );
end

% aggregate the results and display them
show_figure_4;




% Produce Figure 8
% ===================================================

Nfolds = 28;
checkforold = 1;   % check for existing results files
models = [ ignore cond perfect ];
dataset = SEM_visible;
samples_unseen_action = -2;    % don't show any training examples of the unseen action

% run all the experiments needed
for action = 1:Nfolds
    main( models, dataset, Nfolds, checkforold, samples_unseen_action, action );
end

% aggregate the results and display them
show_figure_8;

